(* ::Package:: *)

(*
 *******************************************************
	Package: ImpulsePlot
	Author:  Janos Karsai
	Address: Dept. Medical Informatics,
		 University of Szeged
	         H-6720, Szeged, Hungary
	E-mail:  karsai@dmi.u-szeged.hu
 *******************************************************
*)




(*
*******************************************************
Visualization of impulses

(* Upgraded to Version 6: 2008. Janos Karsai  *)

* Development is supported by 
* Foundation for the Hungarian Higher Education Research, Grant No. 776/94
* Foundation for the Development of Higher Education Grant No. FEFA-IV 1608/4.
* Hungarian Science Research Foundation OTKA T-034275 
* Hungarian Ministry of Education: Project o. 179/1996
* Pro Renovanda Cultura Hungarie: "Science in Education"
* Hungarian Ministry of Education: Higher Education Textbook Grant, 2001
* Hungarian National Foundation for Scientific Research Grant No. T 049516
* INTERREG IIIA Community Initiative Programme Grant No. HUSER0602/066S.
*
*******************************************************
*)


(*
Impulses at fixed instants in 2D
*)

(* Fixed Impulse 1D case  *)


FixedImpulseFieldPlot[Imp_, tn_List, NN_, 
   {x_, x0_, x1_, dx_}, opt___] := ListVectorFieldPlot[
   Flatten[Table[{{tn[[j]], x}, Prepend[Imp[j,tn,{x}]-{x}, 0]}, 
     {x, x0, x1, dx}, {j, 1, NN}], 1], 
   ColorFunction -> (RGBColor[0, 0, 0.8] & ), ScaleFactor -> None, opt]

FixedImpulseFieldPlot[Imp_, tn_List, {n1_,n2_}, 
   {x_, x0_, x1_, dx_}, opt___] := ListVectorFieldPlot[
   Flatten[Table[{{tn[[j]], x}, Prepend[Imp[j,tn,{x}]-{x}, 0]}, 
     {x, x0, x1, dx}, {j, n1, n2}], 1], 
   ColorFunction -> (RGBColor[0, 0, 0.8] & ), ScaleFactor -> None, opt]

FixedImpulseFieldPlot[Imp_, tn_List, {t_,t0_,t1_}, 
   {x_, x0_, x1_, dx_}, opt___] := ListVectorFieldPlot[
   Flatten[Table[{{tn[[j]], x}, Prepend[Imp[j,tn,{x}]-{x}, 0]}, 
     {x, x0, x1, dx}, 
	{j, First[Position[tn,u_/;u>=t0]][[1]], Last[Position[tn,u_/;u<=t1]][[1]]}], 1], 
   ColorFunction -> (RGBColor[0, 0, 0.8] & ), ScaleFactor -> None, AxesLabel->{t,x}
	,Ticks -> {tn, Automatic}, opt]


AnimateImpulse[Imp_, tn_,  NN_, {x_, x0_, x1_}, opt___] :=
 Table[Plot[{x, Imp[j,tn,{x}]}, {x, x0, x1}, 
     AspectRatio -> Automatic, PlotStyle ->{{RGBColor[0, 0, 0]}, 
	{Thickness[0.015], RGBColor[0, 0, 1]}}, AxesLabel -> {x, "Impulse"}, 
     PlotLabel -> StringJoin["Impulse at t=", ToString[tn[[j]]]],opt],
   {j, 1, NN}];

AnimateImpulse[Imp_, tn_,{t_,t0_,t1_}, {x_, x0_, x1_}, opt___] :=
 Table[Plot[{x, Imp[j,tn,{x}]}, {x, x0, x1}, 
     AspectRatio -> Automatic, PlotStyle ->{{RGBColor[0, 0, 0]}, 
	{Thickness[0.015], RGBColor[0, 0, 1]}}, AxesLabel -> {x, "Impulse"}, 
     PlotLabel -> StringJoin["Impulse at t=", ToString[tn[[j]]]],opt],
   {j, First[Position[tn,u_/;u>=t0]][[1]], Last[Position[tn,u_/;u<=t1]][[1]]}];


StackImpulse[Imp_, tn_, NN_, {x_, x0_, x1_}, opt___] := 
 Graphics3D[
		Table[Plot[{x, Imp[j, tn, {x}]}, {x, x0, x1}, 
         AspectRatio -> Automatic, PlotStyle -> 
          {{RGBColor[0, 0, 0]}, {Thickness[0.005], 
            RGBColor[0, 0, 1]}}][[1]]
             /. Line[uu_] :> Line[(Prepend[#1, tn[[j]]] & ) /@ 
          uu], {j, 1, NN}], opt,AxesLabel -> {t, x, "Impulse"}, 
    Ticks -> {tn, Automatic, Automatic}, Axes -> True]; 

StackImpulse[Imp_, tn_, {t_,t0_,t1_}, {x_, x0_, x1_}, opt___] := 
 Graphics3D[
		Table[Plot[{x, Imp[j, tn, {x}]}, {x, x0, x1}, 
         AspectRatio -> Automatic, PlotStyle -> 
          {{RGBColor[0, 0, 0]}, {Thickness[0.005], 
            RGBColor[0, 0, 1]}}][[1]]
             /. Line[uu_] :> Line[(Prepend[#1, tn[[j]]] & ) /@ 
          uu], {j, First[Position[tn,u_/;u>=t0]][[1]], Last[Position[tn,u_/;u<=t1]][[1]]}]
	, opt,AxesLabel -> {t, x, "Impulse"}, 
    Ticks -> {tn, Automatic, Automatic}, Axes -> True]; 

ContourFieldPlot2D[surf_, field_List, {x_, x0_, x1_}, {y_, y0_, y1_},
{ContourOpt_, FieldOpt_, ShowOpt_}] := 
   Module[{pp}, pp = ContourPlot[surf, {x, x0, x1}, {y, y0, y1}, 
       Evaluate[Sequence @@ ContourOpt], ContourShading -> False, 
       Contours -> {0}, Axes -> True, PlotPoints -> 20]; 
		Show[pp, 
		ListVectorFieldPlot[
       (Map[{#1, field /. x :> #1[[1]] /. y :> #1[[2]]} &,
		Flatten[Cases[Flatten[Normal[pp][[1]]],Line[u_List],2]/.Line[u_]->u,1]]), 
       Sequence @@ FieldOpt], 
      Sequence @@ ShowOpt]]; 


(* Fixed Impulse 2D case, Autonomous case  *)

Remove[DoubleFieldPlot]
DoubleFieldPlot[{fld1_,fld2_},{x_, x0_, x1_}, {y_, y0_, y1_}, opt___] := 
 Show[
	VectorFieldPlot[fld1,{x,x0,x1},{y,y0,y1},ColorFunction->None,opt],
	VectorFieldPlot[fld2,{x,x0,x1},{y,y0,y1},
	ScaleFactor->None,ColorFunction->Hue,opt]
,Axes->True];


(*
Impulses at fixed instants in 3D
*)

(* Fixed Impulse 1D case  *)


Remove[FixedImpulseFieldPlot3D]
FixedImpulseFieldPlot3D[Imp_, tn_List, NN_, {x_, x0_, x1_, dx_}, {y_, y0_, y1_, dy_}, opt___] := 
  ListVectorFieldPlot3D[
	Flatten[ Table[{{tn[[i]], x, y}, Prepend[Imp[i,tn,{x,y}] - {x, y}, 0]}, 
     {x, x0, x1, dx}, {y, y0, y1, dy}, {i, 1, NN}], 2], 
   ScaleFactor -> None, opt, VectorHeads -> True, 
	Ticks -> {tn, Automatic, Automatic},
   Axes -> True];

FixedImpulseFieldPlot3D[Imp_, tn_List, {n1_,n2_}, {x_, x0_, x1_, dx_}, {y_, y0_, y1_, dy_}, opt___] := 
  ListVectorFieldPlot3D[
	Flatten[ 
		Table[Map[{{#, x, y}, Prepend[Imp[#,tn,{x,y}] - {x, y}, 0]}&,Take[tn,{n1,n2}]],
     {x, x0, x1, dx}, {y, y0, y1, dy}], 2], 
   ScaleFactor -> None, opt, VectorHeads -> True,
	Ticks -> {tn, Automatic, Automatic},
   Axes -> True];

FixedImpulseFieldPlot3D[Imp_,tn_List,{t_,t0_,t1_},{x_,x0_,x1_,dx_},{y_,y0_,y1_,dy_},opt___]:=
 ListVectorFieldPlot3D[
	Flatten[ 
		Table[Map[{{#, x, y}, Prepend[Imp[#,tn,{x,y}] - {x, y}, 0]}&,
		Take[tn,{First[Position[tn,u_/;u>=t0]][[1]],Last[Position[tn,u_/;u<=t1]][[1]]}]],
     {x, x0, x1, dx}, {y, y0, y1, dy}], 2], AxesLabel->{t,x,y},
   ScaleFactor -> None, opt, VectorHeads -> True,
	Ticks -> {tn, Automatic, Automatic},
   Axes -> True];


(* General Impulses 2D case  *)



Clear[JumpPlot]
JumpPlot[{SS_, II_}, {t_, t0_, t1_}, 
     {x_, x0_, x1_}, opt___] := 
    Module[{pp},Show[pp = ContourPlot[SS, {t, t0, t1}, 
       {x, x0, x1}, Contours -> {0}, opt, 
       AspectRatio -> Automatic, Axes -> True, 
       PlotPoints -> 100, ContourShading -> 
        False, AxesLabel -> {t, x}, 
       ContourStyle -> {Thickness[0.007]}], 
     Graphics[Normal[pp[[1]]] /. 
       Line[uu_] :> {RGBColor[0.8, 0, 0], 
         Line[(Prepend[II /. x :> #1[[2]] /. 
              t :> #1[[1]], #1[[1]]] & ) /@ 
           uu]}]]
		]


JumpPlot[{SS_, II_,d_/;d===0 ||d===1}, {t_, t0_, t1_}, 
     {x_, x0_, x1_}, opt___] := 
    Module[{pp},Show[pp = ContourPlot[SS, {t, t0, t1}, 
       {x, x0, x1}, Contours -> {0}, opt, 
       AspectRatio -> Automatic, Axes -> True, 
       PlotPoints -> 100, ContourShading -> 
        False, AxesLabel -> {t, x}, 
       ContourStyle -> {Thickness[0.005],RGBColor[0,0,0.7d]}], 
     Graphics[Normal[pp[[1]]] /. 
       Line[uu_] :> {RGBColor[0.8, 0, 0], 
         Line[(Prepend[II /. x :> #1[[2]] /. 
              t :> #1[[1]], #1[[1]]] & ) /@ 
           uu]}]]
	]


(* General Impulses 3D case  *)



Clear[JumpPlot3D]
JumpPlot3D[{SS_, II_}, {t_, t0_, t1_}, {x_, x0_, x1_}, 
     {y_, y0_, y1_}, opt___] := 
    Module[{pp},
Show[pp = ContourPlot3D[SS, {t, t0, t1}, {x, x0, x1}, 
       {y, y0, y1}, Contours -> {0}, opt, Axes -> True,  
       AxesLabel -> {t, x, y}, 
		ContourStyle -> Directive[Red,Opacity[0.6],Thickness[0.007]]],
		pp/.{GraphicsComplex[u_,v___]:>
		GraphicsComplex[Map[(Prepend[II /. t :> #1[[1]] /. x :> #1[[2]] /. 
              y :> #1[[3]], #1[[1]]]) &,u],v]} /.{Red->Blue,Opacity[_]->Opacity[0.8]}
]];


JumpPlot3D[{SS_, II_,d_/;d===0 ||d===1}, {t_, t0_, t1_}, {x_, x0_, x1_}, 
     {y_, y0_, y1_}, opt___] := 
    Module[{pp},
	Show[
		pp = ContourPlot3D[SS, {t, t0, t1}, {x, x0, x1}, 
       {y, y0, y1}, Contours -> {0}, opt, Axes -> True,  
       AxesLabel -> {t, x, y}, 
		ContourStyle -> Directive[Red,Opacity[0.6],Thickness[0.007]]],
		pp/.{GraphicsComplex[u_,v___]:>
		GraphicsComplex[Map[(Prepend[II /. t :> #1[[1]] /. x :> #1[[2]] /. 
              y :> #1[[3]], #1[[1]]]) &,u],v]} /.{Red->Blue,Opacity[_]->Opacity[0.8]}
]];


(* Autonomous Impulses 2D case  *)



Clear[AutonomousJumpPlot2D]
AutonomousJumpPlot2D[{SS_, II_}, {x_, x0_, x1_}, 
     {y_, y0_, y1_}, opt___] := 
    Module[{pp},Show[pp = ContourPlot[SS, {x, x0, x1}, {y, y0, y1}, 
       Contours -> {0}, Axes -> True, 
       ContourShading -> False, AxesLabel -> {x, y}, 
        opt,ContourStyle -> {Thickness[0.007]}], 
	Graphics[Normal[pp[[1]]] /. 
       Line[uu_] :> {RGBColor[0.9, 0, 0], 
         Line[(II /. x :> #1[[1]] /. y :> #1[[2]] & ) /@ uu]}],
	Graphics[Normal[pp[[1]]] /. Line[uu_] :> 
        (Arrow[{#1, II /. x :> #1[[1]] /. y :> #1[[2]]}] & ) /@ 
         uu]]
]


AutonomousJumpPlot2D[{SS_, II_,d_/;d===0 ||d===1}, {x_, x0_, x1_}, 
     {y_, y0_, y1_}, opt___] := 
    Module[{pp},Show[pp = ContourPlot[SS, {x, x0, x1}, {y, y0, y1}, 
       Contours -> {0}, Axes -> True, 
       ContourShading -> False, AxesLabel -> {x, y}, 
        opt,ContourStyle -> {Thickness[0.005],RGBColor[0,0,0.7d]}], 
Graphics[Normal[pp[[1]]] /. 
       Line[uu_] :> {RGBColor[0.9, 0, 0], 
         Line[(II /. x :> #1[[1]] /. y :> #1[[2]] & ) /@ uu]}],
Graphics[Normal[pp[[1]]] /. Line[uu_] :> 
        (Arrow[{#1, II /. x :> #1[[1]] /. y :> #1[[2]]}] & ) /@ 
         uu]]]


(* Autonomous Impulses 3D case  *)



Remove[AutonomousJumpPlot3D]
AutonomousJumpPlot3D[{SS_, II_}, {x_, x0_, x1_}, 
     {y_, y0_, y1_}, {z_, z0_, z1_}, opt___] := 
    Module[{pp},Show[
	pp = ContourPlot3D[SS, {x, x0, x1}, {y, y0, y1}, 
       {z, z0, z1}, Contours -> {0}, opt, Axes -> True, 
       PlotPoints -> 4, AxesLabel -> {x, y, z},ContourStyle -> Directive[Blue,Opacity[0.6]]],
pp/.{GraphicsComplex[u_,v___]:>
		GraphicsComplex[Map[(II /. x :> #1[[1]] /. y :> #1[[2]] /. 
             z:> #1[[3]]) &,u],v]} /.{Blue->Red,Opacity[0.6]->Opacity[0.8]},
ListVectorFieldPlot3D[pp[[1]]/.GraphicsComplex[u_,v___]:> Map[{#,-#+II /. x :> #1[[1]] /. y :> #1[[2]] /. 
             z:> #1[[3]]} &,u],ScaleFactor->None]
 ]]


AutonomousJumpPlot3D[{SS_, II_,d_/;d===0 ||d===1}, {x_, x0_, x1_}, 
     {y_, y0_, y1_}, {z_, z0_, z1_}, opt___] := 
    Module[{pp},Show[
	pp = ContourPlot3D[SS, {x, x0, x1}, {y, y0, y1}, 
       {z, z0, z1}, Contours -> {0}, opt, Axes -> True, 
       PlotPoints -> 4, AxesLabel -> {x, y, z},ContourStyle -> Directive[Blue,Opacity[0.6]]],
	pp/.{GraphicsComplex[u_,v___]:>
		GraphicsComplex[Map[(II /. x :> #1[[1]] /. y :> #1[[2]] /. 
             z:> #1[[3]]) &,u],v]} /.{Blue->Red,Opacity[0.6]->Opacity[0.8]},
	ListVectorFieldPlot3D[pp[[1]]/.GraphicsComplex[u_,v___]:> Map[{#,-#+II /. x :> #1[[1]] /. y :> #1[[2]] /. 
             z:> #1[[3]]} &,u],ScaleFactor->None]
 ]]
