(* ::Package:: *)

(*
 *******************************************************
	Package: ODESolve
	Author:  Janos Karsai
	Address: Dept. Medical Informatics,
			 University of Szeged
	         H-6720, Szeged, Hungary
	E-mail:  karsai@dmi.u-szeged.hu
 *******************************************************
*)

(*
Package for solution and visualization of differential equations

Version: Mathematica 6.0
2008.
*)


(*
*******************************************************
* Development is supported by 
* Foundation for the Hungarian Higher Education Research, Grant No. 776/94
* Foundation for the Development of Higher Education Grant No. FEFA-IV 1608/4.
* Hungarian Science Research Foundation OTKA T-034275 
* Hungarian Ministry of Education: Project o. 179/1996
* Pro Renovanda Cultura Hungarie: "Science in Education"
* Hungarian Ministry of Education: Higher Education Textbook Grant, 2001
* Hungarian National Foundation for Scientific Research Grant No. T 049516
* INTERREG IIIA Community Initiative Programme Grant No. HUSER0602/066S.
*******************************************************
*)


BeginPackage["`ODESolve`"]; 

 
ODESolve0::usage =  
"ODESolve0[{rhs1,rhs2,...},{y1,y2,...},{y10,y20,...},{t,t0,T},opt]
solves the ordinary differential system on the interval [t0,T]. 
NDSolve with opt options is used. The right hand
side of the system is {rhs1,rhs2,...} without the arguments, the dependent 
variables are {y1,y2,...}, The function gives the solution
in the list of InterpolationFunctions with argument.";

ODESolve::usage = 
"ODESolve[{rhs1,rhs2,...},{y1,y2,...},ICTabl,{t,t0,T},opt] solves
the ordinary differential system with multiple set of initial conditions.
on the interval [t0,T]. NDSolve with opt options is used. The right hand
side of the system is {rhs1,rhs2,...}, the dependent 
variables are {y1,y2,...}, ICTabl is a list of initial conditions of form
{{y1-0,y2-0,...}},...,{y1-K,y20-K,...}}
The function gives the solution in the list of InterpolationFunctions with 
argument t.";

ODEICGen::usage = "Gives the equation with initial condition for ODESolve. ";

ODEGen::usage ="Gives the ODE from its right had side";

ICGen::usage ="Gives the intial contions of form x[t0]==x0...";

FGen::usage ="Gives the form x[t] from x";


Begin["`Private`"];


ODEGen[vec_List, var_List,t_] := 
  MapThread[
    Equal, {var /. Map[ (# -> #'[t]) &, var], 
      vec /. (Map[ (# -> #[t]) &, var])}];

ICGen[IC_, t0_, var_] := 
  Table[var[[i]][t0] == IC[[i]], {i, 1, Length[var]}];

FGen[var_,t_] := var /. (Map[ (# -> #[t]) &, var]);

ODEICGen[vec_List,IC_List, {t0_,t_},var_List]:=Join[ODEGen[vec, var,t],ICGen[IC, t0, var]];


(*
  eqn. solution (InterpolationFunction object)
*)


ODESolve0[xdot_List,xvar_List,icond_List,
		{t_,t0_,T_},opt___]:=
Module[{},
Return[(xvar /. Map[#->#[t]&,#]& [xvar]) /.
First[NDSolve[
	Join[
	Table[
	 xvar[[i]]'[t]==
         (xdot[[i]]/. Map[#->#[t]&,#]& [xvar]),{i,1,Length[xvar]}],
	 Table[xvar[[i]][t0]==icond[[i]],{i,1,Length[xvar]}]],xvar,
	{t,t0,T},opt]
    ]
]];

(*
Solve with a table of init. conditions (table of Interpol.Funct. object)
*)

ODESolve[xdot_List,xvar_List,ictab_List,{t_,t0_,T_},opt___]:=
	Map[(ODESolve0[xdot,xvar,#,{t,t0,T},opt])&,ictab];

(* Create projection of the solutions with the given variables *)
	
End[];

EndPackage[]; 

